# Author(s):
# - Cameron Rutherford <cameron.rutherford@pnnl.gov>

include( "${CMAKE_CURRENT_LIST_DIR}/HiOpTargets.cmake" )

# Configure TPLs if not already in an export set

# If using MPI, make sure module is loaded
if(ON AND NOT TARGET MPI::MPI_CXX)  
  find_package(MPI REQUIRED)
  include_directories( )
  if(NOT DEFINED MPI_CXX_COMPILER)
    set(CMAKE_CXX_COMPILER /opt/local/bin/mpicxx-mpich-mp)
    set(CMAKE_C_COMPILER /opt/local/bin/mpicc-mpich-mp)
  endif()
  include_directories( )
endif()

if(OFF AND NOT TARGET OpenMP::OpenMP_CXX)
  find_package(OpenMP)
endif()

# Need to export RAJA and umpire as they have find_package
if(OFF AND NOT TARGET RAJA)
  find_package(RAJA PATHS )
  find_package(umpire PATHS )
endif()

set(HiOp::SPARSE ON CACHE BOOL "indicates if hiop sparse is enabled")

if(OFF AND NOT TARGET Ginkgo)
  find_package(Ginkgo PATHS )
endif()

if(OFF)
  if(OFF)
    include(CheckLanguage)
    # This must come before enable_language(CUDA)
    set(CMAKE_CUDA_COMPILER )
    enable_language(CUDA)
    check_language(CUDA)
    set(CMAKE_CUDA_FLAGS "")
    find_package(CUDAToolkit REQUIRED)
  elseif(OFF)
    find_package(hip REQUIRED)
    find_package(hipfft REQUIRED)
    find_package(hiprand REQUIRED)
    find_package(rocrand REQUIRED)
    find_package(hipblas REQUIRED)
    find_package(rocblas REQUIRED)
    find_package(hipcub REQUIRED)
    find_package(rocprim REQUIRED)
    add_definitions(-DHAVE_HIP) # For hipmagma
  endif()
endif()

# Add HiOp::HiOp alias library
if(ON AND NOT TARGET HiOp::HiOp)
  add_library(HiOp::HiOp ALIAS HiOp::hiop_shared)
endif()

if(ON AND NOT TARGET HiOp::HiOp)
  add_library(HiOp::HiOp ALIAS HiOp::hiop_shared)
endif()

